@@ -47,8 +47,10 @@ module GetText
         end
         default_path_rules += DEFAULT_RULES
 
-        load_path = $LOAD_PATH.dup
-        load_path.map!{|v| v.match(/(.*?)(\/lib)*?$/); $1}
+        load_path = $LOAD_PATH.map {|path|
+          path = path.to_path if path.respond_to?(:to_path)
+          path.gsub(/\/lib\z/, "")
+        }
         load_path.each {|path|
           default_path_rules += [
             "#{path}/data/locale/%{lang}/LC_MESSAGES/%{name}.mo",
