@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2012-2014  Kouhei Sutou <kou@clear-code.com>
+# Copyright (C) 2012-2015  Kouhei Sutou <kou@clear-code.com>
 # Copyright (C) 2010  Eddie Lau <tatonlto@gmail.com>
 #
 # License: Ruby's or LGPL
@@ -218,6 +218,31 @@ msgstr "bonjour"
         merged_po = merge
         assert_equal(pot_creation_date, merged_po[""].msgstr)
       end
+
+      def test_already_fuzzy_po
+        @po["hello"] = generate_entry(:msgid => "hello",
+                                      :msgstr => "bonjour",
+                                      :flag => "fuzzy")
+        @pot["hello"] = generate_entry(:msgid => "hello",
+                                       :msgstr => nil)
+        @pot["helol"] = generate_entry(:msgid => "helol",
+                                       :msgstr => nil,
+                                       :flag => "fuzzy")
+        merged_po = merge
+        assert_true(merged_po.has_key?("helol"))
+        assert_equal(["fuzzy"], merged_po["helol"].flags)
+      end
+
+      def test_already_fuzzy_pot
+        @po["hello"] = generate_entry(:msgid => "hello",
+                                      :msgstr => "bonjour")
+        @pot["helol"] = generate_entry(:msgid => "helol",
+                                       :msgstr => nil,
+                                       :flag => "fuzzy")
+        merged_po = merge
+        assert_true(merged_po.has_key?("helol"))
+        assert_equal(["fuzzy"], merged_po["helol"].flags)
+      end
     end
 
     class TestAddFuzzy < self
@@ -423,9 +448,10 @@ EOP
     end
 
     class TestFuzzy < self
-      def test_header_message
-        @msgmerge.run("--update", @po_file_path, @pot_file_path)
-        assert_equal(<<-EOP, File.read(@po_file_path))
+      class TestHeader < self
+        def test_remove_fuzzy_from_pot
+          @msgmerge.run("--update", @po_file_path, @pot_file_path)
+          assert_equal(<<-EOP, File.read(@po_file_path))
 #{po_header(@pot_formatted_time, @po_formatted_time)}
 #: hello.rb:1
 msgid "Hello"
@@ -439,6 +465,38 @@ msgstr "Translated World"
 msgid "Hello World"
 msgstr ""
 EOP
+        end
+      end
+
+      class TestMessage < self
+        def po_content
+          <<-PO
+#{po_header(@po_formatted_time, @po_formatted_time)}
+
+#, fuzzy
+msgid "World"
+msgstr "Translated World"
+         PO
+        end
+
+        def test_merge_metadata
+          @msgmerge.run("--update", @po_file_path, @pot_file_path)
+          assert_equal(<<-PO, File.read(@po_file_path))
+#{po_header(@pot_formatted_time, @po_formatted_time)}
+#: hello.rb:1
+msgid "Hello"
+msgstr ""
+
+#: hello.rb:2
+#, fuzzy
+msgid "World"
+msgstr "Translated World"
+
+#: hello.rb:3
+msgid "Hello World"
+msgstr ""
+          PO
+        end
       end
     end
 
